package java.net;

public abstract class Authenticator {
   private static Authenticator theAuthenticator;
   private InetAddress requestingSite;
   private int requestingPort;
   private String requestingProtocol;
   private String requestingPrompt;
   private String requestingScheme;

   private void reset() {
      this.requestingSite = null;
      this.requestingPort = -1;
      this.requestingProtocol = null;
      this.requestingPrompt = null;
      this.requestingScheme = null;
   }

   public static synchronized void setDefault(Authenticator var0) {
      NetPermission var1 = new NetPermission("setDefaultAuthenticator");
      SecurityManager var2 = System.getSecurityManager();
      if (var2 != null) {
         var2.checkPermission(var1);
      }

      if (theAuthenticator == null) {
         theAuthenticator = var0;
      }
   }

   public static PasswordAuthentication requestPasswordAuthentication(InetAddress var0, int var1, String var2, String var3, String var4) {
      NetPermission var5 = new NetPermission("requestPasswordAuthentication");
      SecurityManager var6 = System.getSecurityManager();
      if (var6 != null) {
         var6.checkPermission(var5);
      }

      Authenticator var7 = theAuthenticator;
      if (var7 == null) {
         return null;
      } else {
         synchronized(var7) {
            var7.reset();
            var7.requestingSite = var0;
            var7.requestingPort = var1;
            var7.requestingProtocol = var2;
            var7.requestingPrompt = var3;
            var7.requestingScheme = var4;
            PasswordAuthentication var9 = var7.getPasswordAuthentication();
            return var9;
         }
      }
   }

   protected final InetAddress getRequestingSite() {
      return this.requestingSite;
   }

   protected final int getRequestingPort() {
      return this.requestingPort;
   }

   protected final String getRequestingProtocol() {
      return this.requestingProtocol;
   }

   protected final String getRequestingPrompt() {
      return this.requestingPrompt;
   }

   protected final String getRequestingScheme() {
      return this.requestingScheme;
   }

   protected PasswordAuthentication getPasswordAuthentication() {
      return null;
   }
}
