package java.net;

import java.io.IOException;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;

public class DatagramSocket {
   DatagramSocketImpl impl;
   boolean connected;
   InetAddress connectedAddress;
   int connectedPort;
   static Class implClass;
   static DatagramSocketImplFactory factory;

   public DatagramSocket() throws SocketException {
      this.connected = false;
      this.connectedAddress = null;
      this.connectedPort = -1;
      this.create(0, (InetAddress)null);
   }

   public DatagramSocket(int var1) throws SocketException {
      this(var1, (InetAddress)null);
   }

   public DatagramSocket(int var1, InetAddress var2) throws SocketException {
      this.connected = false;
      this.connectedAddress = null;
      this.connectedPort = -1;
      if (var1 >= 0 && var1 <= 65535) {
         this.create(var1, var2);
      } else {
         throw new IllegalArgumentException("Port out of range:" + var1);
      }
   }

   void create(int var1, InetAddress var2) throws SocketException {
      SecurityManager var3 = System.getSecurityManager();
      if (var3 != null) {
         var3.checkListen(var1);
      }

      if (factory != null) {
         this.impl = factory.createDatagramSocketImpl();
      } else {
         try {
            this.impl = (DatagramSocketImpl)implClass.newInstance();
         } catch (Exception var5) {
            throw new SocketException("can't instantiate DatagramSocketImpl");
         }
      }

      this.impl.create();
      if (var2 == null) {
         var2 = InetAddress.anyLocalAddress;
      }

      this.impl.bind(var1, var2);
   }

   public void connect(InetAddress var1, int var2) {
      synchronized(this) {
         if (var2 >= 0 && var2 <= 65535) {
            if (var1 == null) {
               throw new IllegalArgumentException("connect: null address");
            } else {
               SecurityManager var4 = System.getSecurityManager();
               if (var4 != null) {
                  if (var1.isMulticastAddress()) {
                     var4.checkMulticast(var1);
                  } else {
                     var4.checkConnect(var1.getHostAddress(), var2);
                     var4.checkAccept(var1.getHostAddress(), var2);
                  }
               }

               this.connectedAddress = var1;
               this.connectedPort = var2;
               this.connected = true;
            }
         } else {
            throw new IllegalArgumentException("connect: " + var2);
         }
      }
   }

   public void disconnect() {
      synchronized(this) {
         this.connectedAddress = null;
         this.connectedPort = -1;
         this.connected = false;
      }
   }

   public InetAddress getInetAddress() {
      return this.connectedAddress;
   }

   public int getPort() {
      return this.connectedPort;
   }

   public void send(DatagramPacket var1) throws IOException {
      Object var2 = null;
      synchronized(var1) {
         if (!this.connected) {
            SecurityManager var4 = System.getSecurityManager();
            if (var4 != null) {
               if (var1.getAddress().isMulticastAddress()) {
                  var4.checkMulticast(var1.getAddress());
               } else {
                  var4.checkConnect(var1.getAddress().getHostAddress(), var1.getPort());
               }
            }
         } else {
            InetAddress var7 = var1.getAddress();
            if (var7 == null) {
               var1.setAddress(this.connectedAddress);
               var1.setPort(this.connectedPort);
            } else if (!var7.equals(this.connectedAddress) || var1.getPort() != this.connectedPort) {
               throw new IllegalArgumentException("connected address and packet address differ");
            }
         }

         this.impl.send(var1);
      }
   }

   public synchronized void receive(DatagramPacket var1) throws IOException {
      Object var2 = null;
      synchronized(var1) {
         SecurityManager var11;
         if (this.connected || (var11 = System.getSecurityManager()) != null) {
            while(true) {
               InetAddress var4 = new InetAddress();
               int var5 = this.impl.peek(var4);
               if (!this.connected) {
                  if (var11 != null) {
                     try {
                        var11.checkAccept(var4.getHostAddress(), var5);
                        break;
                     } catch (SecurityException var9) {
                        DatagramPacket var7 = new DatagramPacket(new byte[1], 1);
                        this.impl.receive(var7);
                     }
                  }
               } else {
                  if (this.connectedAddress.equals(var4) && this.connectedPort == var5) {
                     break;
                  }

                  DatagramPacket var6 = new DatagramPacket(new byte[1], 1);
                  this.impl.receive(var6);
               }
            }
         }

         this.impl.receive(var1);
      }
   }

   public InetAddress getLocalAddress() {
      Object var1 = null;

      try {
         var4 = (InetAddress)this.impl.getOption(15);
         SecurityManager var2 = System.getSecurityManager();
         if (var2 != null) {
            var2.checkConnect(var4.getHostAddress(), -1);
         }
      } catch (Exception var3) {
         var4 = InetAddress.anyLocalAddress;
      }

      return var4;
   }

   public int getLocalPort() {
      return this.impl.getLocalPort();
   }

   public synchronized void setSoTimeout(int var1) throws SocketException {
      this.impl.setOption(4102, new Integer(var1));
   }

   public synchronized int getSoTimeout() throws SocketException {
      Object var1 = this.impl.getOption(4102);
      return var1 instanceof Integer ? (Integer)var1 : 0;
   }

   public synchronized void setSendBufferSize(int var1) throws SocketException {
      if (var1 <= 0) {
         throw new IllegalArgumentException("negative send size");
      } else {
         this.impl.setOption(4097, new Integer(var1));
      }
   }

   public synchronized int getSendBufferSize() throws SocketException {
      int var1 = 0;
      Object var2 = this.impl.getOption(4097);
      if (var2 instanceof Integer) {
         var1 = (Integer)var2;
      }

      return var1;
   }

   public synchronized void setReceiveBufferSize(int var1) throws SocketException {
      if (var1 <= 0) {
         throw new IllegalArgumentException("invalid receive size");
      } else {
         this.impl.setOption(4098, new Integer(var1));
      }
   }

   public synchronized int getReceiveBufferSize() throws SocketException {
      int var1 = 0;
      Object var2 = this.impl.getOption(4098);
      if (var2 instanceof Integer) {
         var1 = (Integer)var2;
      }

      return var1;
   }

   public void close() {
      this.impl.close();
   }

   public static synchronized void setDatagramSocketImplFactory(DatagramSocketImplFactory var0) throws IOException {
      if (factory != null) {
         throw new SocketException("factory already defined");
      } else {
         SecurityManager var1 = System.getSecurityManager();
         if (var1 != null) {
            var1.checkSetFactory();
         }

         factory = var0;
      }
   }

   static {
      String var0 = "";

      try {
         var0 = (String)AccessController.doPrivileged(new GetPropertyAction("impl.prefix", "Plain"));
         implClass = Class.forName("java.net." + var0 + "DatagramSocketImpl");
      } catch (Exception var3) {
         System.err.println("Can't find class: java.net." + var0 + "DatagramSocketImpl: check impl.prefix property");
      }

      if (implClass == null) {
         try {
            implClass = Class.forName("java.net.PlainDatagramSocketImpl");
         } catch (Exception var2) {
            throw new Error("System property impl.prefix incorrect");
         }
      }

   }
}
