package java.net;

import java.io.FileDescriptor;
import java.io.IOException;
import java.security.AccessController;
import sun.security.action.LoadLibraryAction;

class PlainDatagramSocketImpl extends DatagramSocketImpl {
   private int timeout = 0;

   protected synchronized void create() throws SocketException {
      super.fd = new FileDescriptor();
      this.datagramSocketCreate();
   }

   protected synchronized native void bind(int var1, InetAddress var2) throws SocketException;

   protected native void send(DatagramPacket var1) throws IOException;

   protected synchronized native int peek(InetAddress var1) throws IOException;

   protected synchronized native void receive(DatagramPacket var1) throws IOException;

   protected native void setTimeToLive(int var1) throws IOException;

   protected native int getTimeToLive() throws IOException;

   protected native void setTTL(byte var1) throws IOException;

   protected native byte getTTL() throws IOException;

   protected native void join(InetAddress var1) throws IOException;

   protected native void leave(InetAddress var1) throws IOException;

   protected void close() {
      if (super.fd != null) {
         this.datagramSocketClose();
         super.fd = null;
      }

   }

   protected void finalize() {
      this.close();
   }

   public void setOption(int var1, Object var2) throws SocketException {
      switch (var1) {
         case 4:
            if (var2 == null || !(var2 instanceof Integer)) {
               throw new SocketException("bad argument for SO_REUSEADDR");
            }
            break;
         case 15:
            throw new SocketException("Cannot re-bind Socket");
         case 16:
            if (var2 == null || !(var2 instanceof InetAddress)) {
               throw new SocketException("bad argument for IP_MULTICAST_IF");
            }
            break;
         case 4097:
         case 4098:
            if (var2 == null || !(var2 instanceof Integer) || (Integer)var2 < 0) {
               throw new SocketException("bad argument for SO_SNDBUF or SO_RCVBUF");
            }
            break;
         case 4102:
            if (var2 != null && var2 instanceof Integer) {
               int var3 = (Integer)var2;
               if (var3 < 0) {
                  throw new IllegalArgumentException("timeout < 0");
               }

               this.timeout = var3;
               return;
            }

            throw new SocketException("bad argument for SO_TIMEOUT");
         default:
            throw new SocketException("invalid option: " + var1);
      }

      this.socketSetOption(var1, var2);
   }

   public Object getOption(int var1) throws SocketException {
      Integer var2 = null;
      if (var1 == 4102) {
         var2 = new Integer(this.timeout);
      } else {
         int var3 = this.socketGetOption(var1);
         if (var1 == 15 || var1 == 16) {
            InetAddress var4 = new InetAddress();
            var4.address = var3;
            return var4;
         }

         if (var1 == 4098 || var1 == 4097) {
            var2 = new Integer(var3);
         }
      }

      return var2;
   }

   private native void datagramSocketCreate() throws SocketException;

   private native void datagramSocketClose();

   private native void socketSetOption(int var1, Object var2) throws SocketException;

   private native int socketGetOption(int var1) throws SocketException;

   private static native void init();

   static {
      AccessController.doPrivileged(new LoadLibraryAction("net"));
      init();
   }
}
