package java.net;

import java.io.FileDescriptor;
import java.io.IOException;

public class ServerSocket {
   private SocketImpl impl;
   private static SocketImplFactory factory;

   private ServerSocket() throws IOException {
      this.impl = (SocketImpl)(factory != null ? factory.createSocketImpl() : new PlainSocketImpl());
   }

   public ServerSocket(int var1) throws IOException {
      this(var1, 50, (InetAddress)null);
   }

   public ServerSocket(int var1, int var2) throws IOException {
      this(var1, var2, (InetAddress)null);
   }

   public ServerSocket(int var1, int var2, InetAddress var3) throws IOException {
      this();
      if (var1 >= 0 && var1 <= 65535) {
         try {
            SecurityManager var4 = System.getSecurityManager();
            if (var4 != null) {
               var4.checkListen(var1);
            }

            this.impl.create(true);
            if (var3 == null) {
               var3 = InetAddress.anyLocalAddress;
            }

            this.impl.bind(var3, var1);
            this.impl.listen(var2);
         } catch (SecurityException var6) {
            this.impl.close();
            throw var6;
         } catch (IOException var7) {
            this.impl.close();
            throw var7;
         }
      } else {
         throw new IllegalArgumentException("Port value out of range: " + var1);
      }
   }

   public InetAddress getInetAddress() {
      return this.impl.getInetAddress();
   }

   public int getLocalPort() {
      return this.impl.getLocalPort();
   }

   public Socket accept() throws IOException {
      Socket var1 = new Socket();
      this.implAccept(var1);
      return var1;
   }

   protected final void implAccept(Socket var1) throws IOException {
      SocketImpl var2 = var1.impl;

      try {
         var1.impl = null;
         var2.address = new InetAddress();
         var2.fd = new FileDescriptor();
         this.impl.accept(var2);
         SecurityManager var3 = System.getSecurityManager();
         if (var3 != null) {
            var3.checkAccept(var2.getInetAddress().getHostAddress(), var2.getPort());
         }
      } catch (IOException var10) {
         var2.reset();
         throw var10;
      } catch (SecurityException var11) {
         var2.reset();
         throw var11;
      } finally {
         var1.impl = var2;
      }

   }

   public void close() throws IOException {
      this.impl.close();
   }

   public synchronized void setSoTimeout(int var1) throws SocketException {
      this.impl.setOption(4102, new Integer(var1));
   }

   public synchronized int getSoTimeout() throws IOException {
      Object var1 = this.impl.getOption(4102);
      return var1 instanceof Integer ? (Integer)var1 : 0;
   }

   public String toString() {
      return "ServerSocket[addr=" + this.impl.getInetAddress() + ",port=" + this.impl.getPort() + ",localport=" + this.impl.getLocalPort() + "]";
   }

   public static synchronized void setSocketFactory(SocketImplFactory var0) throws IOException {
      if (factory != null) {
         throw new SocketException("factory already defined");
      } else {
         SecurityManager var1 = System.getSecurityManager();
         if (var1 != null) {
            var1.checkSetFactory();
         }

         factory = var0;
      }
   }
}
