package java.net;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.AccessController;
import java.security.AllPermission;
import java.security.Permission;
import java.util.Date;
import java.util.Hashtable;
import java.util.StringTokenizer;
import sun.net.www.MimeTable;
import sun.security.action.GetPropertyAction;

public abstract class URLConnection {
   protected URL url;
   protected boolean doInput = true;
   protected boolean doOutput = false;
   private static boolean defaultAllowUserInteraction = false;
   protected boolean allowUserInteraction;
   private static boolean defaultUseCaches = true;
   protected boolean useCaches;
   protected long ifModifiedSince;
   protected boolean connected;
   private static FileNameMap fileNameMap;
   private static boolean fileNameMapLoaded = false;
   static ContentHandlerFactory factory;
   private static Hashtable handlers = new Hashtable();
   private static final ContentHandler UnknownContentHandlerP = new UnknownContentHandler();
   private static final String contentClassPrefix = "sun.net.www.content";
   private static final String contentPathProp = "java.content.handler.pkgs";

   public static synchronized FileNameMap getFileNameMap() {
      if (fileNameMap == null && !fileNameMapLoaded) {
         fileNameMap = MimeTable.loadTable();
         fileNameMapLoaded = true;
      }

      return new 1();
   }

   public static void setFileNameMap(FileNameMap var0) {
      SecurityManager var1 = System.getSecurityManager();
      if (var1 != null) {
         var1.checkSetFactory();
      }

      fileNameMap = var0;
   }

   public abstract void connect() throws IOException;

   protected URLConnection(URL var1) {
      this.allowUserInteraction = defaultAllowUserInteraction;
      this.useCaches = defaultUseCaches;
      this.ifModifiedSince = 0L;
      this.connected = false;
      this.url = var1;
   }

   public URL getURL() {
      return this.url;
   }

   public int getContentLength() {
      return this.getHeaderFieldInt("content-length", -1);
   }

   public String getContentType() {
      return this.getHeaderField("content-type");
   }

   public String getContentEncoding() {
      return this.getHeaderField("content-encoding");
   }

   public long getExpiration() {
      return this.getHeaderFieldDate("expires", 0L);
   }

   public long getDate() {
      return this.getHeaderFieldDate("date", 0L);
   }

   public long getLastModified() {
      return this.getHeaderFieldDate("last-modified", 0L);
   }

   public String getHeaderField(String var1) {
      return null;
   }

   public int getHeaderFieldInt(String var1, int var2) {
      try {
         return Integer.parseInt(this.getHeaderField(var1));
      } catch (Throwable var4) {
         return var2;
      }
   }

   public long getHeaderFieldDate(String var1, long var2) {
      try {
         return Date.parse(this.getHeaderField(var1));
      } catch (Throwable var5) {
         return var2;
      }
   }

   public String getHeaderFieldKey(int var1) {
      return null;
   }

   public String getHeaderField(int var1) {
      return null;
   }

   public Object getContent() throws IOException {
      this.getInputStream();
      return this.getContentHandler().getContent(this);
   }

   public Object getContent(Class[] var1) throws IOException {
      this.getInputStream();
      return this.getContentHandler().getContent(this, var1);
   }

   public Permission getPermission() throws IOException {
      return new AllPermission();
   }

   public InputStream getInputStream() throws IOException {
      throw new UnknownServiceException("protocol doesn't support input");
   }

   public OutputStream getOutputStream() throws IOException {
      throw new UnknownServiceException("protocol doesn't support output");
   }

   public String toString() {
      return this.getClass().getName() + ":" + this.url;
   }

   public void setDoInput(boolean var1) {
      if (this.connected) {
         throw new IllegalAccessError("Already connected");
      } else {
         this.doInput = var1;
      }
   }

   public boolean getDoInput() {
      return this.doInput;
   }

   public void setDoOutput(boolean var1) {
      if (this.connected) {
         throw new IllegalAccessError("Already connected");
      } else {
         this.doOutput = var1;
      }
   }

   public boolean getDoOutput() {
      return this.doOutput;
   }

   public void setAllowUserInteraction(boolean var1) {
      if (this.connected) {
         throw new IllegalAccessError("Already connected");
      } else {
         this.allowUserInteraction = var1;
      }
   }

   public boolean getAllowUserInteraction() {
      return this.allowUserInteraction;
   }

   public static void setDefaultAllowUserInteraction(boolean var0) {
      defaultAllowUserInteraction = var0;
   }

   public static boolean getDefaultAllowUserInteraction() {
      return defaultAllowUserInteraction;
   }

   public void setUseCaches(boolean var1) {
      if (this.connected) {
         throw new IllegalAccessError("Already connected");
      } else {
         this.useCaches = var1;
      }
   }

   public boolean getUseCaches() {
      return this.useCaches;
   }

   public void setIfModifiedSince(long var1) {
      if (this.connected) {
         throw new IllegalAccessError("Already connected");
      } else {
         this.ifModifiedSince = var1;
      }
   }

   public long getIfModifiedSince() {
      return this.ifModifiedSince;
   }

   public boolean getDefaultUseCaches() {
      return defaultUseCaches;
   }

   public void setDefaultUseCaches(boolean var1) {
      defaultUseCaches = var1;
   }

   public void setRequestProperty(String var1, String var2) {
      if (this.connected) {
         throw new IllegalAccessError("Already connected");
      }
   }

   public String getRequestProperty(String var1) {
      if (this.connected) {
         throw new IllegalAccessError("Already connected");
      } else {
         return null;
      }
   }

   public static void setDefaultRequestProperty(String var0, String var1) {
   }

   public static String getDefaultRequestProperty(String var0) {
      return null;
   }

   public static synchronized void setContentHandlerFactory(ContentHandlerFactory var0) {
      if (factory != null) {
         throw new Error("factory already defined");
      } else {
         SecurityManager var1 = System.getSecurityManager();
         if (var1 != null) {
            var1.checkSetFactory();
         }

         factory = var0;
      }
   }

   synchronized ContentHandler getContentHandler() throws UnknownServiceException {
      String var1 = this.stripOffParameters(this.getContentType());
      ContentHandler var2 = null;
      if (var1 == null) {
         throw new UnknownServiceException("no content-type");
      } else {
         try {
            var2 = (ContentHandler)handlers.get(var1);
            if (var2 != null) {
               return var2;
            }
         } catch (Exception var5) {
         }

         if (factory != null) {
            var2 = factory.createContentHandler(var1);
         }

         if (var2 == null) {
            try {
               var2 = this.lookupContentHandlerClassFor(var1);
            } catch (Exception var4) {
               ((Throwable)var4).printStackTrace();
               var2 = UnknownContentHandlerP;
            }

            handlers.put(var1, var2);
         }

         return var2;
      }
   }

   private String stripOffParameters(String var1) {
      int var2 = var1.indexOf(59);
      return var2 > 0 ? var1.substring(0, var2) : var1;
   }

   private ContentHandler lookupContentHandlerClassFor(String var1) throws InstantiationException, IllegalAccessException, ClassNotFoundException {
      String var2 = this.typeToPackageName(var1);
      String var3 = this.getContentHandlerPkgPrefixes();
      StringTokenizer var4 = new StringTokenizer(var3, "|");

      while(var4.hasMoreTokens()) {
         String var5 = var4.nextToken().trim();

         try {
            String var6 = var5 + "." + var2;
            Class var7 = null;

            try {
               var7 = Class.forName(var6);
            } catch (ClassNotFoundException var10) {
               ClassLoader var9 = ClassLoader.getSystemClassLoader();
               if (var9 != null) {
                  var7 = var9.loadClass(var6);
               }
            }

            if (var7 != null) {
               ContentHandler var8 = (ContentHandler)var7.newInstance();
               return var8;
            }
         } catch (Exception var11) {
         }
      }

      return UnknownContentHandlerP;
   }

   private String typeToPackageName(String var1) {
      var1 = var1.toLowerCase();
      int var2 = var1.length();
      char[] var3 = new char[var2];
      var1.getChars(0, var2, var3, 0);

      for(int var4 = 0; var4 < var2; ++var4) {
         char var5 = var3[var4];
         if (var5 == '/') {
            var3[var4] = '.';
         } else if (('A' > var5 || var5 > 'Z') && ('a' > var5 || var5 > 'z') && ('0' > var5 || var5 > '9')) {
            var3[var4] = '_';
         }
      }

      return new String(var3);
   }

   private String getContentHandlerPkgPrefixes() {
      String var1 = (String)AccessController.doPrivileged(new GetPropertyAction("java.content.handler.pkgs", ""));
      if (var1 != "") {
         var1 = var1 + "|";
      }

      return var1 + "sun.net.www.content";
   }

   protected static String guessContentTypeFromName(String var0) {
      Object var1 = null;
      String var2 = getFileNameMap().getContentTypeFor(var0);
      return var2;
   }

   public static String guessContentTypeFromStream(InputStream var0) throws IOException {
      var0.mark(10);
      int var1 = var0.read();
      int var2 = var0.read();
      int var3 = var0.read();
      int var4 = var0.read();
      int var5 = var0.read();
      int var6 = var0.read();
      int var7 = var0.read();
      int var8 = var0.read();
      var0.reset();
      if (var1 == 202 && var2 == 254 && var3 == 186 && var4 == 190) {
         return "application/java-vm";
      } else if (var1 == 172 && var2 == 237) {
         return "application/x-java-serialized-object";
      } else if (var1 == 71 && var2 == 73 && var3 == 70 && var4 == 56) {
         return "image/gif";
      } else if (var1 == 35 && var2 == 100 && var3 == 101 && var4 == 102) {
         return "image/x-bitmap";
      } else if (var1 == 33 && var2 == 32 && var3 == 88 && var4 == 80 && var5 == 77 && var6 == 50) {
         return "image/x-pixmap";
      } else if (var1 == 137 && var2 == 80 && var3 == 78 && var4 == 71 && var5 == 13 && var6 == 10 && var7 == 26 && var8 == 10) {
         return "image/png";
      } else if (var1 == 46 && var2 == 115 && var3 == 110 && var4 == 100) {
         return "audio/basic";
      } else if (var1 == 100 && var2 == 110 && var3 == 115 && var4 == 46) {
         return "audio/basic";
      } else {
         if (var1 == 60) {
            if (var2 == 33 || var2 == 104 && (var3 == 116 && var4 == 109 && var5 == 108 || var3 == 101 && var4 == 97 && var5 == 100) || var2 == 98 && var3 == 111 && var4 == 100 && var5 == 121 || var2 == 72 && (var3 == 84 && var4 == 77 && var5 == 76 || var3 == 69 && var4 == 65 && var5 == 68) || var2 == 66 && var3 == 79 && var4 == 68 && var5 == 89) {
               return "text/html";
            }

            if (var2 == 63 && var3 == 120 && var4 == 109 && var5 == 108 && var6 == 32) {
               return "application/xml";
            }
         }

         if (var1 == 254 && var2 == 255 && var3 == 0 && var4 == 60 && var5 == 0 && var6 == 63 && var7 == 0 && var8 == 120) {
            return "application/xml";
         } else if (var1 == 255 && var2 == 254 && var3 == 60 && var4 == 0 && var5 == 63 && var6 == 0 && var7 == 120 && var8 == 0) {
            return "application/xml";
         } else if (var1 == 255 && var2 == 216 && var3 == 255 && var4 == 224) {
            return "image/jpeg";
         } else if (var1 == 255 && var2 == 216 && var3 == 255 && var4 == 238) {
            return "image/jpg";
         } else if (var1 == 82 && var2 == 73 && var3 == 70 && var4 == 70) {
            return "audio/x-wav";
         } else if (var1 == 208 && var2 == 207 && var3 == 17 && var4 == 224 && var5 == 161 && var6 == 177 && var7 == 26 && var8 == 225 && checkfpx(var0)) {
            return "image/vnd.fpx";
         } else {
            return null;
         }
      }
   }

   private static boolean checkfpx(InputStream var0) throws IOException {
      var0.mark(256);
      long var1 = 28L;

      long var3;
      for(var3 = 0L; var3 != var1; var3 += var0.skip(var1 - var3)) {
      }

      int var7 = var0.read();
      var0.read();
      long var5 = var3 + 2L;
      int var19;
      if (var7 == 254) {
         var19 = var0.read();
         var19 += var0.read() << 8;
      } else {
         var19 = var0.read() << 8;
         var19 += var0.read();
      }

      var5 += 2L;
      var1 = 48L - var5;

      for(var3 = 0L; var3 != var1; var3 += var0.skip(var1 - var3)) {
      }

      var5 += var3;
      int var23;
      if (var7 == 254) {
         var23 = var0.read();
         var23 += var0.read() << 8;
         var23 += var0.read() << 16;
         var23 += var0.read() << 24;
      } else {
         var23 = var0.read() << 24;
         var23 += var0.read() << 16;
         var23 += var0.read() << 8;
         var23 += var0.read();
      }

      var5 += 4L;
      var0.reset();
      var1 = 512L + (long)(1 << var19) * (long)var23 + 80L;
      var0.mark((int)var1 + 48);

      for(long var15 = 0L; var15 != var1; var15 += var0.skip(var1 - var15)) {
      }

      int[] var10 = new int[16];

      for(int var11 = 0; var11 < 16; ++var11) {
         var10[var11] = var0.read();
      }

      if (var7 == 254 && var10[0] == 0 && var10[2] == 97 && var10[3] == 86 && var10[4] == 84 && var10[5] == 193 && var10[6] == 206 && var10[7] == 17 && var10[8] == 133 && var10[9] == 83 && var10[10] == 0 && var10[11] == 170 && var10[12] == 0 && var10[13] == 161 && var10[14] == 249 && var10[15] == 91) {
         var0.reset();
         return true;
      } else if (var10[3] == 0 && var10[1] == 97 && var10[0] == 86 && var10[5] == 84 && var10[4] == 193 && var10[7] == 206 && var10[6] == 17 && var10[8] == 133 && var10[9] == 83 && var10[10] == 0 && var10[11] == 170 && var10[12] == 0 && var10[13] == 161 && var10[14] == 249 && var10[15] == 91) {
         var0.reset();
         return true;
      } else {
         var0.reset();
         return false;
      }
   }

   // $FF: synthetic method
   static FileNameMap access$000() {
      return fileNameMap;
   }
}
